@@ -17,8 +17,8 @@ func TestStatusLine(t *testing.T) {
 }
 
 func testStatusLine(t *testing.T, statusCode int, expected []byte) {
-	line := statusLine(statusCode)
+	line := formatStatusLine(nil, strHTTP11, statusCode, s2b(StatusMessage(statusCode)))
 	if !bytes.Equal(expected, line) {
-		t.Fatalf("unexpected status line %s. Expecting %s", string(line), string(expected))
+		t.Fatalf("unexpected status line %q. Expecting %q", string(line), string(expected))
 	}
 }
